import re

def postgisToSpatialite(sql, targetSrid='2154'):
    '''
    Convert postgis SQL statement
    into spatialite compatible
    statements
    '''

    # delete some incompatible options
    # replace other by spatialite syntax
    replaceDict = [
        # delete
        {'in': r'with\(oids=.+\)', 'out': ''},
        {'in': r'comment on [^;]+;', 'out': ''},
        {'in': r'alter table ([^;]+) add primary key( )+\(([^;]+)\);',
        'out': r'create index idx_\1_\3 on \1 (\3);'},
        {'in': r'alter table ([^;]+) add constraint [^;]+ primary key( )+\(([^;]+)\);',
        'out': r'create index idx_\1_\3 on \1 (\3);'},
        {'in': r'alter table [^;]+drop column[^;]+;', 'out': ''},
        {'in': r'alter table [^;]+drop constraint[^;]+;', 'out': ''},
        #~ {'in': r'^analyse [^;]+;', 'out': ''},
        # replace
        {'in': r'truncate (bati|fanr|lloc|nbat|pdll|prop)',
        'out': r'drop table \1;create table \1 (tmp text)'},
        {'in': r'truncate ', 'out': 'delete from '},
        {'in': r'distinct on *\([a-z, ]+\)', 'out': 'distinct'},
        {'in': r'serial', 'out': 'INTEGER PRIMARY KEY AUTOINCREMENT'},
        {'in': r'string_agg', 'out': 'group_concat'},
        {'in': r'current_schema::text, ', 'out': ''},
        {'in': r'substring', 'out': 'SUBSTR'},
        {'in': r"(to_char\()([^']+) *, *'[09]+' *\)", 'out': r"CAST(\2 AS TEXT)"},
        {'in': r"(to_number\()([^']+) *, *'[09]+' *\)", 'out': r"CAST(\2 AS float)"},
        {'in': r"(to_date\()([^']+) *, *'DDMMYYYY' *\)",
        'out': r"date(substr(\2, 5, 4) || '-' || substr(\2, 3, 2) || '-' || substr(\2, 1, 2))"},
        {'in': r"(to_date\()([^']+) *, *'DD/MM/YYYY' *\)",
        'out': r"date(substr(\2, 7, 4) || '-' || substr(\2, 4, 2) || '-' || substr(\2, 1, 2))"},
        {'in': r"(to_date\()([^']+) *, *'YYYYMMDD' *\)",
        'out': r"date(substr(\2, 1, 4) || '-' || substr(\2, 5, 2) || '-' || substr(\2, 7, 2))"},
        {'in': r"(to_char\()([^']+) *, *'dd/mm/YYYY' *\)",
        'out': r"strftime('%d/%m/%Y', \2)"},
        {'in': r"ST_MakeValid\(geom\)",
         'out': r"CASE WHEN ST_IsValid(geom) THEN geom ELSE ST_Buffer(geom,0) END"},
        {'in': r"ST_MakeValid\(p\.geom\)",
         'out': r"CASE WHEN ST_IsValid(p.geom) THEN p.geom ELSE ST_Buffer(p.geom,0) END"}
    ]

    for a in replaceDict:
        r = re.compile(a['in'], re.IGNORECASE|re.MULTILINE)
        sql = r.sub(a['out'], sql)
        #self.updateLog(sql)

    # index spatiaux
    r = re.compile(r'(create index [^;]+ ON )([^;]+)( USING +)(gist +)?\(([^;]+)\);',  re.IGNORECASE|re.MULTILINE)
    sql = r.sub(r"SELECT createSpatialIndex('\2', '\5');", sql)

    # replace postgresql "update from" statement
    r = re.compile(r'(update [^;=]+)(=)([^;=]+ FROM [^;]+)(;)', re.IGNORECASE|re.MULTILINE)
    sql = r.sub(r'\1=(SELECT \3);', sql)

    #self.updateLog(sql)
    return sql
